from typing import Dict, Tuple
from lib.parsing.parser import Parser

parser = Parser()
examples: Dict[Tuple[str, str], Tuple[str, str]] = {}
convert_examples: Dict[str, str] = {}

teacher_dfa = "(0,a)=1 (1,b)=0 final={0}"
student_dfa = "(A,a)=B (B,b)=C (C,a)=B final={A,C}"
teacher_dfa_total = parser.dfa_to_str(parser.str_to_dfa(teacher_dfa).total())
teacher_efa = "(A,a)={B} (A,\e)={D} (B,b)={A} final={A,D}"
student_nfa = parser.nfa_to_str(parser.str_to_nfa(teacher_efa).eliminate_epsilon())
student_gra = "S -> aS' | \e\nS' -> bA | b\nA -> aS'"
student_reg = "(ab)^*"
student_efa = "(A,a)={B} (A,\e)={D} (B,b)={C} (C,a)={B} final={C,D}"

examples[('DFA', 'DFA')] = (teacher_dfa, student_dfa)
examples[('DFA', 'TOT')] = (teacher_dfa, teacher_dfa_total)
examples[('DFA', 'MIN')] = (teacher_dfa, teacher_dfa_total)
examples[('DFA', 'EFA')] = (teacher_dfa, student_efa)
examples[('DFA', 'NFA')] = (teacher_dfa, student_nfa)
examples[('DFA', 'GRA')] = (teacher_dfa, student_gra)
examples[('DFA', 'REG')] = (teacher_dfa, student_reg)

examples[('EFA', 'DFA')] = (student_gra, student_dfa)
examples[('EFA', 'TOT')] = (student_gra, teacher_dfa_total)
examples[('EFA', 'MIN')] = (student_gra, teacher_dfa_total)
examples[('EFA', 'EFA')] = (student_gra, student_efa)
examples[('EFA', 'NFA')] = (student_gra, student_nfa)
examples[('EFA', 'GRA')] = (student_gra, student_gra)
examples[('EFA', 'REG')] = (student_gra, student_reg)

examples[('GRA', 'DFA')] = (student_reg, student_dfa)
examples[('GRA', 'TOT')] = (student_reg, teacher_dfa_total)
examples[('GRA', 'MIN')] = (student_reg, teacher_dfa_total)
examples[('GRA', 'EFA')] = (student_reg, student_efa)
examples[('GRA', 'NFA')] = (student_reg, student_nfa)
examples[('GRA', 'GRA')] = (student_reg, student_gra)
examples[('GRA', 'REG')] = (student_reg, student_reg)

examples[('REG', 'DFA')] = (student_reg, student_dfa)
examples[('REG', 'TOT')] = (student_reg, teacher_dfa_total)
examples[('REG', 'MIN')] = (student_reg, teacher_dfa_total)
examples[('REG', 'EFA')] = (student_reg, student_efa)
examples[('REG', 'NFA')] = (student_reg, student_nfa)
examples[('REG', 'GRA')] = (student_reg, student_gra)
examples[('REG', 'REG')] = (student_reg, student_reg)

convert_examples['DFA'] = teacher_dfa
convert_examples['EFA'] = student_nfa
convert_examples['GRA'] = student_gra
convert_examples['REG'] = student_reg