from typing import Set, Tuple, Dict, Union
from lib.common import Terminal, Nonterminal, State, Character, Eps, Emptyset
from lib.grammars import RegGrammar, CFG
from lib.dfa import DFA
from lib.nfa import NFA
from lib.regex import RegEx


class RegGrammar(RegGrammar):
    def eliminate_useless(self):
        grammar = super().eliminate_useless() # TODO ask how to do this more efficiently
        return RegGrammar(grammar.nonterminals, grammar.terminals, grammar.rules, grammar.init)
        

class Grammar(CFG):
    pass


class DFA(DFA):
    def dfa_to_nfa(self) -> NFA:
        new_transition: NFA.transition = {}
        for (state, character) in self.transition:
            new_transition[state, character] = {self.transition[state, character]}
        return NFA(self.states, self.characters, new_transition, self.init, self.final)


class NFA(NFA):
    def eliminate_epsilon(self) -> NFA:
        nfa = super().eliminate_epsilon() # TODO ask how to do this more efficiently
        return NFA(nfa.states, nfa.characters, nfa.transition, nfa.init, nfa.final)

    def nfa_to_reggrammar(self) -> RegGrammar:
        nonterminals: Set[Nonterminal] = set()
        terminals: Set[Terminal] = set()
        rules: RegGrammar.Rules = dict()
        init = Nonterminal(self.init.name)

        for state in self.states:
            nonterminals.add(Nonterminal(state.name))
        for character in self.characters:
            terminals.add(Terminal(character.name))

        for state, character in self.transition:
            for dest_state in self.transition[state, character]:
                nonterminal1 = Nonterminal(state.name)
                nonterminal2 = Nonterminal(dest_state.name)
                terminal = Terminal(character.name)
                rules.setdefault(nonterminal1, set()).add((terminal, nonterminal2))

                if dest_state in self.final:
                    rules[nonterminal1].add(terminal)

        if self.init in self.final:
            new_init = Nonterminal("<newinit>")
            rules[new_init] = {Eps()}
            for rule in rules[init]:
                rules[new_init].add(rule)
            init = new_init
            nonterminals.add(init)

        return RegGrammar(nonterminals, terminals, rules, init)

    def nfa_to_regex(self) -> RegEx:
        # I don't want to have to do this really don't
        pass

class RegEx(RegEx):
    def regex_to_efa(self) -> NFA:
        nfa = super().regex_to_efa()  # TODO ask how to do this more efficiently
        return NFA(nfa.states, nfa.characters, nfa.transition, nfa.init, nfa.final)
